@@ -7,13 +7,18 @@ import os, sys
 BUILD_LANGS = [
     'en', # base strings
     'de',
+    'eo',
     'es',
+    'es_MX',
     'fi',
     'fr',
+    'gl',
     'ia',
     'ie',
+    'isv',
     'pl',
     'ru',
+    'sk',
     'sr',
     'tok',
     'zh_Hans',
@@ -101,7 +106,9 @@ def parse_po(src):
 def compile_string(msg_id, msg_str):
     return msg_id.encode('utf-8') + bytes([0]) + \
            msg_str.encode('utf-8') + bytes([0])
-                          
+    
+
+os.chdir(os.path.dirname(__file__))
     
 if MODE == 'compile':
     BASE_STRINGS = {}
