@@ -59,6 +59,7 @@ enum iGmDocumentTheme {
     white_GmDocumentTheme,
     sepia_GmDocumentTheme,
     highContrast_GmDocumentTheme,
+    max_GmDocumentTheme
 };
 
 iBool isDark_GmDocumentTheme(enum iGmDocumentTheme);
@@ -174,11 +175,11 @@ enum iGmDocumentUpdate {
 void    setThemeSeed_GmDocument (iGmDocument *, const iBlock *seed);
 void    setFormat_GmDocument    (iGmDocument *, enum iSourceFormat format);
 void    setBanner_GmDocument    (iGmDocument *, enum iGmDocumentBanner type);
-void    setWidth_GmDocument     (iGmDocument *, int width);
+void    setWidth_GmDocument     (iGmDocument *, int width, int outsideMargin);
 void    redoLayout_GmDocument   (iGmDocument *);
 iBool   updateOpenURLs_GmDocument(iGmDocument *);
 void    setUrl_GmDocument       (iGmDocument *, const iString *url);
-void    setSource_GmDocument    (iGmDocument *, const iString *source, int width,
+void    setSource_GmDocument    (iGmDocument *, const iString *source, int width, int outsideMargin,
                                  enum iGmDocumentUpdate updateType);
 void    foldPre_GmDocument      (iGmDocument *, uint16_t preId);
 
