@@ -49,9 +49,8 @@ int main(int argc, char **argv) {
     registerURLHandler_MacOS();
 #endif
 #if defined (iPlatformMsys)
-    /* MSYS runtime takes care of WinMain. */
-    setDPIAware_Win32();
-    SDL_SetMainReady();
+    init_Win32(); /* DPI awareness, dark mode */
+    SDL_SetMainReady(); /* MSYS runtime takes care of WinMain. */
 #endif
     /* Initialize libraries. */
 #if defined (LAGRANGE_ENABLE_MPG123)
@@ -64,13 +63,18 @@ int main(int argc, char **argv) {
                           "ECDHE-ECDSA-AES128-GCM-SHA256:"
                           "ECDHE-RSA-AES256-GCM-SHA384:"
                           "ECDHE-RSA-CHACHA20-POLY1305:"
-                          "ECDHE-RSA-AES128-GCM-SHA256");
+                          "ECDHE-RSA-AES128-GCM-SHA256:"
+                          "DHE-RSA-AES256-GCM-SHA384");
     SDL_SetHint(SDL_HINT_VIDEO_ALLOW_SCREENSAVER, "1");
     SDL_SetHint(SDL_HINT_MAC_CTRL_CLICK_EMULATE_RIGHT_CLICK, "1");
-    if (SDL_Init(SDL_INIT_VIDEO | SDL_INIT_AUDIO | SDL_INIT_TIMER)) {
-        fprintf(stderr, "SDL init failed: %s\n", SDL_GetError());
+    SDL_SetHint(SDL_HINT_VIDEO_X11_NET_WM_BYPASS_COMPOSITOR, "0");
+    if (SDL_Init(SDL_INIT_VIDEO | SDL_INIT_TIMER)) {
+        fprintf(stderr, "[SDL] init failed: %s\n", SDL_GetError());
         return -1;
     }
+    if (SDL_Init(SDL_INIT_AUDIO)) {
+        fprintf(stderr, "[SDL] audio init failed: %s\n", SDL_GetError());
+    }
     run_App(argc, argv);
     SDL_Quit();
 #if defined (LAGRANGE_ENABLE_MPG123)
