@@ -44,11 +44,13 @@ struct Impl_LabelWidget {
     iString command;
     iClick  click;
     struct {
-        uint8_t alignVisual     : 1; /* align according to visible bounds, not font metrics */
-        uint8_t noAutoMinHeight : 1; /* minimum height is not set automatically */
-        uint8_t drawAsOutline   : 1; /* draw as outline, filled with background color */
-        uint8_t noTopFrame      : 1;
-        uint8_t wrap            : 1;
+        uint8_t alignVisual         : 1; /* align according to visible bounds, not font metrics */
+        uint8_t noAutoMinHeight     : 1; /* minimum height is not set automatically */
+        uint8_t drawAsOutline       : 1; /* draw as outline, filled with background color */
+        uint8_t noTopFrame          : 1;
+        uint8_t wrap                : 1;
+        uint8_t allCaps             : 1;
+        uint8_t removeTrailingColon : 1;
     } flags;
 };
 
@@ -68,7 +70,7 @@ static iInt2 padding_LabelWidget_(const iLabelWidget *d, int corner) {
                              : corner == 1 ? init_I2(w->padding[2], w->padding[1])
                              : corner == 2 ? init_I2(w->padding[2], w->padding[3])
                              : init_I2(w->padding[0], w->padding[3]));
-#if defined (iPlatformAppleMobile)
+#if defined (iPlatformMobile)
     return add_I2(widgetPad,
                   init_I2(flags & tight_WidgetFlag ? 2 * gap_UI : (4 * gap_UI),
                           (flags & extraPadding_WidgetFlag ? 1.5f : 1.0f) * 3 * gap_UI / 2));
@@ -124,6 +126,11 @@ static iBool processEvent_LabelWidget_(iLabelWidget *d, const SDL_Event *ev) {
         updateKey_LabelWidget_(d);
         return iFalse;
     }
+    else if (isCommand_Widget(w, ev, "focus.gained") ||
+             isCommand_Widget(w, ev, "focus.lost")) {
+        refresh_Widget(d);
+        return iFalse;
+    }
     if (!isEmpty_String(&d->command)) {
 #if 0 && defined (iPlatformAppleMobile)
         /* Touch allows activating any button on release. */
@@ -159,10 +166,17 @@ static iBool processEvent_LabelWidget_(iLabelWidget *d, const SDL_Event *ev) {
         switch (ev->type) {
             case SDL_KEYDOWN: {
                 const int mods = ev->key.keysym.mod;
-                if (d->key && ev->key.keysym.sym == d->key && checkModifiers_(mods, d->kmods)) {
+                const int sym  = ev->key.keysym.sym;
+                if (d->key && sym == d->key && checkModifiers_(mods, d->kmods)) {
                     trigger_LabelWidget_(d);
                     return iTrue;
                 }
+                if (isFocused_Widget(d) && mods == 0 &&
+                    (sym == SDLK_RETURN || sym == SDLK_KP_ENTER)) {
+                    trigger_LabelWidget_(d);
+                    refresh_Widget(d);
+                    return iTrue;
+                }
                 break;
             }
         }
@@ -174,14 +188,17 @@ static void keyStr_LabelWidget_(const iLabelWidget *d, iString *str) {
     toString_Sym(d->key, d->kmods, str);
 }
 
-static void getColors_LabelWidget_(const iLabelWidget *d, int *bg, int *fg, int *frame1, int *frame2) {
+static void getColors_LabelWidget_(const iLabelWidget *d, int *bg, int *fg, int *frame1, int *frame2,
+                                   int *icon, int *meta) {
     const iWidget *w           = constAs_Widget(d);
     const int64_t  flags       = flags_Widget(w);
+    const iBool    isFocus     = (flags & focusable_WidgetFlag && isFocused_Widget(d));
     const iBool    isPress     = (flags & pressed_WidgetFlag) != 0;
     const iBool    isSel       = (flags & selected_WidgetFlag) != 0;
     const iBool    isFrameless = (flags & frameless_WidgetFlag) != 0;
     const iBool    isButton    = d->click.button != 0;
     const iBool    isKeyRoot   = (w->root == get_Window()->keyRoot);
+    const iBool    isDarkTheme = isDark_ColorTheme(colorTheme_App());
     /* Default color state. */
     *bg     = isButton && ~flags & noBackground_WidgetFlag ? (d->widget.bgColor != none_ColorId ?
                                                               d->widget.bgColor : uiBackground_ColorId)
@@ -189,8 +206,12 @@ static void getColors_LabelWidget_(const iLabelWidget *d, int *bg, int *fg, int
     *fg     = uiText_ColorId;
     *frame1 = isButton ? uiEmboss1_ColorId : d->widget.frameColor;
     *frame2 = isButton ? uiEmboss2_ColorId : *frame1;
+    *icon   = uiIcon_ColorId;
+    *meta   = uiTextShortcut_ColorId;
     if (flags & disabled_WidgetFlag && isButton) {
-        *fg = uiTextDisabled_ColorId;
+        *icon = uiTextDisabled_ColorId;
+        *fg   = uiTextDisabled_ColorId;
+        *meta = uiTextDisabled_ColorId;
     }
     if (isSel) {
         *bg = uiBackgroundSelected_ColorId;
@@ -210,9 +231,15 @@ static void getColors_LabelWidget_(const iLabelWidget *d, int *bg, int *fg, int
             }
         }
     }
+    if (isFocus) {
+        *frame1 = *frame2 = (isSel ? uiText_ColorId : uiInputFrameFocused_ColorId);
+    }
     int colorEscape = none_ColorId;
     if (startsWith_String(&d->label, "\v")) {
-        colorEscape = cstr_String(&d->label)[1] - asciiBase_ColorEscape; /* TODO: can be two bytes long */
+        colorEscape = parseEscape_Color(cstr_String(&d->label), NULL);
+    }
+    if (colorEscape == uiTextCaution_ColorId) {
+        *icon = *meta = colorEscape;
     }
     if (isHover_LabelWidget_(d)) {
         if (isFrameless) {
@@ -221,42 +248,47 @@ static void getColors_LabelWidget_(const iLabelWidget *d, int *bg, int *fg, int
         }
         else {
             /* Frames matching color escaped text. */
-            if (colorEscape != none_ColorId) {
-                if (isDark_ColorTheme(colorTheme_App())) {
-                    *frame1 = colorEscape;
-                    *frame2 = darker_Color(*frame1);
-                }
-                else {
-                    *bg = *frame1 = *frame2 = colorEscape;
-                    *fg = white_ColorId | permanent_ColorId;
-                }
+            if (colorEscape == uiTextCaution_ColorId) {
+                *frame1 = colorEscape;
+                *frame2 = isDarkTheme ? darker_Color(*frame1) : lighter_Color(*frame1);
             }
             else if (isSel) {
                 *frame1 = uiEmbossSelectedHover1_ColorId;
                 *frame2 = uiEmbossSelectedHover2_ColorId;
             }
             else {
-                if (isButton) *bg = uiBackgroundHover_ColorId;
                 *frame1 = uiEmbossHover1_ColorId;
                 *frame2 = uiEmbossHover2_ColorId;
             }
         }
-    }
-    if (isPress) {
-        *bg = uiBackgroundPressed_ColorId | permanent_ColorId;
-        if (isButton) {
-            *frame1 = uiEmbossPressed1_ColorId;
-            *frame2 = colorEscape != none_ColorId ? colorEscape : uiEmbossPressed2_ColorId;
-        }
-        if (colorEscape == none_ColorId || colorEscape == uiTextAction_ColorId) {
-            *fg = uiTextPressed_ColorId | permanent_ColorId;
-        }
-        else {
-            *fg = isDark_ColorTheme(colorTheme_App()) ? white_ColorId : black_ColorId;
+        if (colorEscape == uiTextCaution_ColorId) {
+            *icon = *meta = *fg = colorEscape;
+            *bg = isDarkTheme ? darker_Color(colorEscape) : lighter_Color(colorEscape);
         }
     }
     if (d->forceFg >= 0) {
-        *fg = d->forceFg;
+        *fg = *icon = *meta = d->forceFg;
+    }
+    if (isPress) {
+        if (colorEscape == uiTextAction_ColorId || colorEscape == uiTextCaution_ColorId) {
+            *bg = colorEscape;
+            *frame1 = *bg;
+            *frame2 = *bg;
+            *fg = *icon = *meta = (isDarkTheme ? black_ColorId : white_ColorId) | permanent_ColorId;
+        }
+        else {
+            *bg = uiBackgroundPressed_ColorId | permanent_ColorId;
+            if (isButton) {
+                *frame1 = uiEmbossPressed1_ColorId;
+                *frame2 = colorEscape != none_ColorId ? colorEscape : uiEmbossPressed2_ColorId;
+            }
+            //if (colorEscape == none_ColorId || colorEscape == uiTextAction_ColorId) {
+            *fg = *icon = *meta = uiTextPressed_ColorId | permanent_ColorId;
+    //        }
+    //        else {
+    //            *fg = (isDark_ColorTheme(colorTheme_App()) ? white_ColorId : black_ColorId) | permanent_ColorId;
+    //        }
+        }
     }
 }
 
@@ -287,13 +319,18 @@ static void draw_LabelWidget_(const iLabelWidget *d) {
     }
     iPaint p;
     init_Paint(&p);
-    int bg, fg, frame, frame2;
-    getColors_LabelWidget_(d, &bg, &fg, &frame, &frame2);
-    const iBool isCaution = startsWith_String(&d->label, uiTextCaution_ColorEscape);
+    int bg, fg, frame, frame2, iconColor, metaColor;
+    getColors_LabelWidget_(d, &bg, &fg, &frame, &frame2, &iconColor, &metaColor);
+    const enum iColorId colorEscape = parseEscape_Color(cstr_String(&d->label), NULL);
+    const iBool isCaution = (colorEscape == uiTextCaution_ColorId);
     if (bg >= 0) {
-        fillRect_Paint(&p, rect, isCaution && isHover ? uiMarked_ColorId : bg);
+        fillRect_Paint(&p, rect, bg);
     }
-    if (~flags & frameless_WidgetFlag) {
+    if (isFocused_Widget(w)) {
+        iRect frameRect = adjusted_Rect(rect, zero_I2(), init1_I2(-1));
+        drawRectThickness_Paint(&p, frameRect, gap_UI / 4, frame);        
+    }
+    else if (~flags & frameless_WidgetFlag) {
         iRect frameRect = adjusted_Rect(rect, zero_I2(), init1_I2(-1));
         if (isButton) {
             iInt2 points[] = {
@@ -310,12 +347,18 @@ static void draw_LabelWidget_(const iLabelWidget *d) {
             }
 #endif
             drawLines_Paint(&p, points + 2, 3, frame2);
-            drawLines_Paint(
-                &p, points, !isHover && flags & d->flags.noTopFrame ? 2 : 3, frame);
+            drawLines_Paint(&p,
+                            points,
+                            isFocused_Widget(w) ? 3 : (!isHover && d->flags.noTopFrame ? 2 : 3),
+                            frame);
         }
     }
     setClip_Paint(&p, rect);
     const int iconPad = iconPadding_LabelWidget_(d);
+//    const int iconColor = isCaution ? uiTextCaution_ColorId
+//                          : flags & (disabled_WidgetFlag | pressed_WidgetFlag) ? fg
+//                          : isHover                                            ? uiIconHover_ColorId
+//                                                                               : uiIcon_ColorId;
     if (d->icon && d->icon != 0x20) { /* no need to draw an empty icon */
         iString str;
         initUnicodeN_String(&str, &d->icon, 1);
@@ -329,16 +372,13 @@ static void draw_LabelWidget_(const iLabelWidget *d) {
                                -gap_UI / 8)),
                 init_I2(iconPad, lineHeight_Text(d->font)) },
             iTrue,
-            isCaution                                            ? uiTextCaution_ColorId
-            : flags & (disabled_WidgetFlag | pressed_WidgetFlag) ? fg
-            : isHover                                            ? uiIconHover_ColorId
-                                                                 : uiIcon_ColorId,
+            iconColor,
             "%s",
             cstr_String(&str));
         deinit_String(&str);
     }
     if (d->flags.wrap) {
-        const iRect cont = contentBounds_LabelWidget_(d); //djusted_Rect(innerBounds_Widget(w), init_I2(iconPad, 0), zero_I2());
+        const iRect cont = contentBounds_LabelWidget_(d);
         drawWrapRange_Text(
             d->font, topLeft_Rect(cont), width_Rect(cont), fg, range_String(&d->label));
     }
@@ -353,9 +393,11 @@ static void draw_LabelWidget_(const iLabelWidget *d) {
                            add_I2(topRight_Rect(bounds),
                                   addX_I2(negX_I2(padding_LabelWidget_(d, 1)),
                                           deviceType_App() == tablet_AppDeviceType ? gap_UI : 0)),
-                           flags & pressed_WidgetFlag ? fg
-                           : isCaution                ? uiTextCaution_ColorId
-                                                      : uiTextShortcut_ColorId,
+                           metaColor,/*
+                           isHover || flags & pressed_WidgetFlag ? fg
+//                           : isCaution                ? uiTextCaution_ColorId
+                           : colorEscape != none_ColorId ? colorEscape
+                                                      : uiTextShortcut_ColorId,*/
                            right_Alignment,
                            cstr_String(&str));
             deinit_String(&str);
@@ -385,7 +427,7 @@ static void draw_LabelWidget_(const iLabelWidget *d) {
         drawCentered_Text(d->font,
                           (iRect){ addX_I2(topRight_Rect(chRect), -iconPad),
                                    init_I2(chSize, height_Rect(chRect)) },
-                          iTrue, uiSeparator_ColorId, rightAngle_Icon);
+                          iTrue, iconColor, rightAngle_Icon);
     }
     unsetClip_Paint(&p);
 }
@@ -442,11 +484,18 @@ void updateSize_LabelWidget(iLabelWidget *d) {
 
 static void replaceVariables_LabelWidget_(iLabelWidget *d) {
     translate_Lang(&d->label);
+    if (d->flags.allCaps) {
+        set_String(&d->label, collect_String(upper_String(&d->label)));
+    }
+    if (d->flags.removeTrailingColon && endsWith_String(&d->label, ":")) {
+        removeEnd_String(&d->label, 1);
+    }
 }
 
 void init_LabelWidget(iLabelWidget *d, const char *label, const char *cmd) {
     iWidget *w = &d->widget;
     init_Widget(w);
+    iZap(d->flags);
     d->font = uiLabel_FontId;
     d->forceFg = none_ColorId;
     d->icon = 0;
@@ -463,12 +512,7 @@ void init_LabelWidget(iLabelWidget *d, const char *label, const char *cmd) {
     d->key   = 0;
     d->kmods = 0;
     init_Click(&d->click, d, !isEmpty_String(&d->command) ? SDL_BUTTON_LEFT : 0);
-    setFlags_Widget(w, hover_WidgetFlag, d->click.button != 0);
-    d->flags.alignVisual     = iFalse;
-    d->flags.noAutoMinHeight = iFalse;
-    d->flags.drawAsOutline   = iFalse;
-    d->flags.noTopFrame      = iFalse;
-    d->flags.wrap            = iFalse;
+    setFlags_Widget(w, focusable_WidgetFlag | hover_WidgetFlag, d->click.button != 0);
     updateSize_LabelWidget(d);
     updateKey_LabelWidget_(d); /* could be bound to another key */
 }
@@ -499,6 +543,14 @@ void setText_LabelWidget(iLabelWidget *d, const iString *text) {
     }
 }
 
+void setTextCStr_LabelWidget(iLabelWidget *d, const char *text) {
+    updateTextCStr_LabelWidget(d, text);
+    updateSize_LabelWidget(d);
+    if (isWrapped_LabelWidget(d)) {
+        sizeChanged_LabelWidget_(d);
+    }
+}
+
 void setAlignVisually_LabelWidget(iLabelWidget *d, iBool alignVisual) {
     d->flags.alignVisual = alignVisual;
 }
@@ -525,6 +577,20 @@ void setOutline_LabelWidget(iLabelWidget *d, iBool drawAsOutline) {
     }
 }
 
+void setAllCaps_LabelWidget(iLabelWidget *d, iBool allCaps) {
+    if (d) {
+        d->flags.allCaps = allCaps;
+        replaceVariables_LabelWidget_(d);
+    }
+}
+
+void setRemoveTrailingColon_LabelWidget(iLabelWidget *d, iBool removeTrailingColon) {
+    if (d) {
+        d->flags.removeTrailingColon = removeTrailingColon;
+        replaceVariables_LabelWidget_(d);
+    }
+}
+
 void updateText_LabelWidget(iLabelWidget *d, const iString *text) {
     set_String(&d->label, text);
     set_String(&d->srcLabel, text);
@@ -533,10 +599,12 @@ void updateText_LabelWidget(iLabelWidget *d, const iString *text) {
 }
 
 void updateTextCStr_LabelWidget(iLabelWidget *d, const char *text) {
-    setCStr_String(&d->label, text);
-    set_String(&d->srcLabel, &d->label);
-    replaceVariables_LabelWidget_(d);
-    refresh_Widget(&d->widget);
+    if (d) {
+        setCStr_String(&d->label, text);
+        set_String(&d->srcLabel, &d->label);
+        replaceVariables_LabelWidget_(d);
+        refresh_Widget(&d->widget);
+    }
 }
 
 void updateTextAndResizeWidthCStr_LabelWidget(iLabelWidget *d, const char *text) {
@@ -544,13 +612,6 @@ void updateTextAndResizeWidthCStr_LabelWidget(iLabelWidget *d, const char *text)
     d->widget.rect.size.x = defaultSize_LabelWidget(d).x;
 }
 
-void setTextCStr_LabelWidget(iLabelWidget *d, const char *text) {
-    setCStr_String(&d->label, text);
-    set_String(&d->srcLabel, &d->label);
-    replaceVariables_LabelWidget_(d);
-    updateSize_LabelWidget(d);
-}
-
 void setCommand_LabelWidget(iLabelWidget *d, const iString *command) {
     set_String(&d->command, command);
 }
@@ -567,19 +628,8 @@ iBool checkIcon_LabelWidget(iLabelWidget *d) {
         d->icon = 0;
         return iFalse;
     }
-    iStringConstIterator iter;
-    init_StringConstIterator(&iter, &d->label);
-    const iChar icon = iter.value;
-    next_StringConstIterator(&iter);
-    if (iter.value == ' ' && icon >= 0x100) {
-        d->icon = icon;
-        remove_Block(&d->label.chars, 0, iter.next - constBegin_String(&d->label));
-        return iTrue;
-    }
-    else {
-        d->icon = 0;
-    }
-    return iFalse;
+    d->icon = removeIconPrefix_String(&d->label);
+    return d->icon != 0;
 }
 
 iChar icon_LabelWidget(const iLabelWidget *d) {
