@@ -628,6 +628,17 @@ static void updateItems_SidebarWidget_(iSidebarWidget *d) {
     updateMouseHover_ListWidget(d->list);
 }
 
+static size_t findItem_SidebarWidget_(const iSidebarWidget *d, int id) {
+    /* Note that this is O(n), so only meant for infrequent use. */
+    for (size_t i = 0; i < numItems_ListWidget(d->list); i++) {
+        const iSidebarItem *item = constItem_ListWidget(d->list, i);
+        if (item->id == id) {
+            return i;
+        }
+    }
+    return iInvalidPos;
+}
+
 static void updateItemHeight_SidebarWidget_(iSidebarWidget *d) {
     if (d->list) {
         const float heights[max_SidebarMode] = { 1.333f, 2.333f, 1.333f, 3.5f, 1.2f };
@@ -865,10 +876,12 @@ static void itemClicked_SidebarWidget_(iSidebarWidget *d, iSidebarItem *item, si
     switch (d->mode) {
         case documentOutline_SidebarMode: {
             const iGmDocument *doc = document_DocumentWidget(document_App());
-            const iGmHeading *head = constAt_Array(headings_GmDocument(doc), item->id);
-            postCommandf_App("document.goto loc:%p", head->text.start);
-            dismissPortraitPhoneSidebars_Root(as_Widget(d)->root);
-            setOpenedFromSidebar_DocumentWidget(document_App(), iTrue);
+            if (item->id < size_Array(headings_GmDocument(doc))) {
+                const iGmHeading *head = constAt_Array(headings_GmDocument(doc), item->id);
+                postCommandf_App("document.goto loc:%p", head->text.start);
+                dismissPortraitPhoneSidebars_Root(as_Widget(d)->root);
+                setOpenedFromSidebar_DocumentWidget(document_App(), iTrue);
+            }
             break;
         }
         case feeds_SidebarMode: {
@@ -1095,21 +1108,24 @@ static iBool handleSidebarCommand_SidebarWidget_(iSidebarWidget *d, const char *
     return iFalse;
 }
 
-static void bookmarkMoved_SidebarWidget_(iSidebarWidget *d, size_t index, size_t beforeIndex) {
+static void bookmarkMoved_SidebarWidget_(iSidebarWidget *d, size_t index, size_t dstIndex,
+                                         iBool isBefore) {
     const iSidebarItem *movingItem = item_ListWidget(d->list, index);
-    const iBool         isLast     = (beforeIndex == numItems_ListWidget(d->list));
+    const iBool         isLast     = (dstIndex == numItems_ListWidget(d->list));
     const iSidebarItem *dstItem    = item_ListWidget(d->list,
                                                      isLast ? numItems_ListWidget(d->list) - 1
-                                                            : beforeIndex);
+                                                            : dstIndex);
+    if (isLast && isBefore) isBefore = iFalse;
     const iBookmark *dst = get_Bookmarks(bookmarks_App(), dstItem->id);
-    if (hasParent_Bookmark(dst, movingItem->id)) {
+    if (hasParent_Bookmark(dst, movingItem->id) || hasTag_Bookmark(dst, remote_BookmarkTag)) {
+        /* Can't move a folder inside itself, and remote bookmarks cannot be reordered. */
         return;
     }
-    reorder_Bookmarks(bookmarks_App(), movingItem->id, dst->order + (isLast ? 1 : 0));
+    reorder_Bookmarks(bookmarks_App(), movingItem->id, dst->order + (isBefore ? 0 : 1));
     get_Bookmarks(bookmarks_App(), movingItem->id)->parentId = dst->parentId;
     updateItems_SidebarWidget_(d);
     /* Don't confuse the user: keep the dragged item in hover state. */
-    setHoverItem_ListWidget(d->list, index < beforeIndex ? beforeIndex - 1 : beforeIndex);
+    setHoverItem_ListWidget(d->list, dstIndex + (isBefore ? 0 : 1) + (index < dstIndex ? -1 : 0));
     postCommandf_App("bookmarks.changed nosidebar:%p", d); /* skip this sidebar since we updated already */
 }
 
@@ -1185,6 +1201,11 @@ static iBool processEvent_SidebarWidget_(iSidebarWidget *d, const SDL_Event *ev)
                                                              d->mode == feeds_SidebarMode)) {
             if (pointerLabel_Command(cmd, "nosidebar") != d) {
                 updateItems_SidebarWidget_(d);
+                if (hasLabel_Command(cmd, "added")) {
+                    const size_t addedId    = argLabel_Command(cmd, "added");
+                    const size_t addedIndex = findItem_SidebarWidget_(d, addedId);
+                    scrollToItem_ListWidget(d->list, addedIndex, 200);
+                }
             }
         }
         else if (equal_Command(cmd, "idents.changed") && d->mode == identities_SidebarMode) {
@@ -1243,17 +1264,19 @@ static iBool processEvent_SidebarWidget_(iSidebarWidget *d, const SDL_Event *ev)
         }
         else if (isCommand_Widget(w, ev, "list.dragged")) {
             iAssert(d->mode == bookmarks_SidebarMode);
-            if (hasLabel_Command(cmd, "before")) {
-                bookmarkMoved_SidebarWidget_(d,
-                                             argU32Label_Command(cmd, "arg"),
-                                             argU32Label_Command(cmd, "before"));
-            }
-            else {
+            if (hasLabel_Command(cmd, "onto")) {
                 /* Dragged onto a folder. */
                 bookmarkMovedOntoFolder_SidebarWidget_(d,
                                                        argU32Label_Command(cmd, "arg"),
                                                        argU32Label_Command(cmd, "onto"));
             }
+            else {
+                const iBool isBefore = hasLabel_Command(cmd, "before");
+                bookmarkMoved_SidebarWidget_(d,
+                                             argU32Label_Command(cmd, "arg"),
+                                             argU32Label_Command(cmd, isBefore ? "before" : "after"),
+                                             isBefore);
+            }
             return iTrue;
         }
         else if (isCommand_Widget(w, ev, "menu.closed")) {
@@ -1442,7 +1465,7 @@ static iBool processEvent_SidebarWidget_(iSidebarWidget *d, const SDL_Event *ev)
                         removeUrl_Visited(vis, url);
                     }
                     else {
-                        visitUrl_Visited(vis, url, transient_VisitedUrlFlag);
+                        visitUrl_Visited(vis, url, transient_VisitedUrlFlag | kept_VisitedUrlFlag);
                     }
                     postCommand_App("visited.changed");
                     return iTrue;
