@@ -35,6 +35,8 @@ enum iFontSize {
     uiMedium_FontSize, /* 1.125 */
     uiBig_FontSize,    /* 1.333 */
     uiLarge_FontSize,  /* 1.666 */
+    uiSmall_FontSize,  /* 0.900 */
+    uiTiny_FontSize,   /* 0.800 */
     contentRegular_FontSize,
     contentMedium_FontSize,
     contentBig_FontSize,
@@ -59,6 +61,7 @@ enum iFontId {
     defaultBig_FontId,
     defaultLarge_FontId,
     defaultSmall_FontId,
+    defaultTiny_FontId,
     /* UI fonts: bold weight */
     defaultBold_FontId,
     defaultMediumBold_FontId,
