@@ -40,6 +40,10 @@ iDeclareType(TouchState)
 #define numHistory_Touch_   5
 #define lastIndex_Touch_    (numHistory_Touch_ - 1)
 
+static const uint32_t longPressSpanMs_  = 500;
+static const uint32_t shortPressSpanMs_ = 250;
+static const int      tapRadiusPt_      = 10;
+
 enum iTouchEdge {
     none_TouchEdge,
     left_TouchEdge,
@@ -55,12 +59,12 @@ enum iTouchAxis {
 struct Impl_Touch {
     SDL_FingerID id;
     iWidget *affinity; /* widget on which the touch started */
-    iWidget *edgeDragging;
     iBool hasMoved;
     iBool isTapBegun;
     iBool isLeftDown;
     iBool isTouchDrag;
     iBool isTapAndHold;
+    iBool didPostEdgeMove;
     iBool didBeginOnTouchDrag;
     int pinchId;
     enum iTouchEdge edge;
@@ -132,9 +136,6 @@ static iTouch *find_TouchState_(iTouchState *d, SDL_FingerID id) {
     return NULL;
 }
 
-static const uint32_t longPressSpanMs_ = 500;
-static const int      tapRadiusPt_     = 10;
-
 iLocalDef float distance_Touch_(const iTouch *d) {
     return length_F3(sub_F3(d->pos[0], d->startPos));
 }
@@ -246,7 +247,14 @@ iLocalDef double accurateTicks_(void) {
     return 1000.0 * (double) count / (double) freq;
 }
 
+static iFloat3 gestureVector_Touch_(const iTouch *d) {
+    const size_t lastIndex = iMin(d->posCount - 1, lastIndex_Touch_);
+    return sub_F3(d->pos[0], d->pos[lastIndex]);
+}
+
 static void update_TouchState_(void *ptr) {
+    iWindow *win = get_Window();
+    const iWidget *oldHover = win->hover;
     iTouchState *d = ptr;
     /* Check for long presses to simulate right clicks. */
     const uint32_t nowTime = SDL_GetTicks();
@@ -255,6 +263,24 @@ static void update_TouchState_(void *ptr) {
         if (touch->pinchId || touch->isTouchDrag) {
             continue;
         }
+        if (touch->edge) {
+            const iFloat3 pos = touch->pos[0];
+            /* Cancel the swipe if the finger doesn't move or moves mostly vertically. */
+            const iFloat3 gestureVector = gestureVector_Touch_(touch);
+            if (fabsf(2 * x_F3(gestureVector)) < fabsf(y_F3(gestureVector)) ||
+                (isStationary_Touch_(touch) && nowTime - touch->startTime > shortPressSpanMs_)) {
+                //const int swipeDir = x_F3(gestureVector) > 0 ? +1 : -1;
+                //dispatchClick_Touch_(touch,
+//                                     touch->edge == left_TouchEdge  && swipeDir > 0 ? SDL_BUTTON_X1 :
+//                                     touch->edge == right_TouchEdge && swipeDir < 0 ? SDL_BUTTON_X2 : 0);
+//                setHover_Widget(NULL);
+                postCommandf_App("edgeswipe.ended abort:1 side:%d id:%llu", touch->edge, touch->id);
+                touch->edge = none_TouchEdge;
+                /* May be a regular drag along the edge so don't remove. */
+                //remove_ArrayIterator(&i);
+            }
+            continue;
+        }
         /* Holding a touch will reset previous momentum for this widget. */
         if (isStationary_Touch_(touch)) {
             const int elapsed = nowTime - touch->startTime;
@@ -266,8 +292,10 @@ static void update_TouchState_(void *ptr) {
             }
             if (elapsed > 50 && !touch->isTapBegun) {
                 /* Looks like a possible tap. */
+                touchState_()->currentTouchPos = initF3_I2(touch->pos[0]);
                 dispatchNotification_Touch_(touch, widgetTapBegins_UserEventCode);
                 dispatchMotion_Touch_(touch->pos[0], 0);
+                refresh_Widget(touch->affinity);
                 touch->isTapBegun = iTrue;
             }
             if (!touch->isTapAndHold && nowTime - touch->startTime >= longPressSpanMs_ &&
@@ -322,6 +350,7 @@ static void update_TouchState_(void *ptr) {
                 setCurrent_Root(mom->affinity->root);
                 dispatchEvent_Widget(mom->affinity, (SDL_Event *) &(SDL_MouseWheelEvent){
                                                         .type = SDL_MOUSEWHEEL,
+                                                        .which = SDL_TOUCH_MOUSEID,
                                                         .timestamp = nowTime,
                                                         .x = pixels.x,
                                                         .y = pixels.y,
@@ -338,8 +367,13 @@ static void update_TouchState_(void *ptr) {
     if (!isEmpty_Array(d->touches) || !isEmpty_Array(d->moms)) {
         addTickerRoot_App(update_TouchState_, NULL, ptr);
     }
+    if (oldHover != win->hover) {
+        refresh_Widget(oldHover);
+        refresh_Widget(win->hover);
+    }
 }
 
+#if 0
 static iWidget *findSlidePanel_Widget_(iWidget *d) {
     for (iWidget *w = d; w; w = parent_Widget(w)) {
         if (isVisible_Widget(w) && flags_Widget(w) & edgeDraggable_WidgetFlag) {
@@ -348,6 +382,7 @@ static iWidget *findSlidePanel_Widget_(iWidget *d) {
     }
     return NULL;
 }
+#endif
 
 static void checkNewPinch_TouchState_(iTouchState *d, iTouch *newTouch) {
     iWidget *affinity = newTouch->affinity;
@@ -365,6 +400,12 @@ static void checkNewPinch_TouchState_(iTouchState *d, iTouch *newTouch) {
         pinch.touchIds[1] = other->id;
         newTouch->pinchId = other->pinchId = pinch.id;
         clearWidgetMomentum_TouchState_(d, affinity);
+        if (other->edge && other->didPostEdgeMove) {
+            postCommandf_App("edgeswipe.ended abort:1 side:%d id:%llu", other->edge, other->id);
+            other->didPostEdgeMove = iFalse;
+        }
+        other->edge = none_TouchEdge;
+        newTouch->edge = none_TouchEdge;
         /* Remember current positions to determine pinch amount. */
         newTouch->startPos = newTouch->pos[0];
         other->startPos    = other->pos[0];
@@ -431,13 +472,9 @@ iBool processEvent_Touch(const SDL_Event *ev) {
     }
     iTouchState *d = touchState_();
     iWindow *window = get_Window();    
-    if (!isFinished_Anim(&window->rootOffset)) {
-        return iFalse;
-    }
     const iInt2 rootSize = size_Window(window);
     const SDL_TouchFingerEvent *fing = &ev->tfinger;
-    const iFloat3 pos = add_F3(init_F3(fing->x * rootSize.x, fing->y * rootSize.y, 0), /* pixels */
-                               init_F3(0, -value_Anim(&window->rootOffset), 0));
+    const iFloat3 pos = init_F3(fing->x * rootSize.x, fing->y * rootSize.y, 0); /* pixels */
     const uint32_t nowTime = SDL_GetTicks();
     if (ev->type == SDL_FINGERDOWN) {
         /* Register the new touch. */
@@ -452,6 +489,7 @@ iBool processEvent_Touch(const SDL_Event *ev) {
             edge = right_TouchEdge;
         }
         iWidget *aff = hitChild_Window(window, init_I2(iRound(x), iRound(y_F3(pos))));
+#if 0
         if (edge == left_TouchEdge) {
             dragging = findSlidePanel_Widget_(aff);
             if (dragging) {
@@ -460,6 +498,7 @@ iBool processEvent_Touch(const SDL_Event *ev) {
                 setFlags_Widget(dragging, dragged_WidgetFlag, iTrue);
             }
         }
+#endif
         /* TODO: We must retain a reference to the affinity widget, or otherwise it might
            be destroyed during the gesture. */
 //        printf("aff:[%p] %s:'%s'\n", aff, aff ? class_Widget(aff)->name : "-",
@@ -469,7 +508,7 @@ iBool processEvent_Touch(const SDL_Event *ev) {
         iTouch newTouch = {
             .id = fing->fingerId,
             .affinity = aff,
-            .edgeDragging = dragging,
+//            .edgeDragging = dragging,
             .didBeginOnTouchDrag = (flags_Widget(aff) & touchDrag_WidgetFlag) != 0,
             .edge = edge,
             .startTime = nowTime,
@@ -487,6 +526,16 @@ iBool processEvent_Touch(const SDL_Event *ev) {
     }
     else if (ev->type == SDL_FINGERMOTION) {
         iTouch *touch = find_TouchState_(d, fing->fingerId);
+        if (touch && touch->edge) {
+            clear_Array(d->moms);
+            pushPos_Touch_(touch, pos, nowTime);
+            postCommandf_App("edgeswipe.moved arg:%d side:%d id:%llu",
+                             (int) (x_F3(pos) - x_F3(touch->startPos)),
+                             touch->edge,
+                             touch->id);
+            touch->didPostEdgeMove = iTrue;
+            return iTrue;
+        }
         if (touch && touch->affinity) {
             if (touch->isTouchDrag) {
                 dispatchMotion_Touch_(pos, SDL_BUTTON_LMASK);
@@ -556,46 +605,29 @@ iBool processEvent_Touch(const SDL_Event *ev) {
                     touch->axis = y_TouchAxis;
                 }
             }
-            /* Edge swipe aborted? */
-            if (touch->edge == left_TouchEdge) {
-                if (fing->dx < 0 && x_F3(touch->pos[0]) < tapRadiusPt_ * window->pixelRatio) {
-                    touch->edge = none_TouchEdge;
-                    if (touch->edgeDragging) {
-                        setFlags_Widget(touch->edgeDragging, dragged_WidgetFlag, iFalse);
-                        setVisualOffset_Widget(touch->edgeDragging, 0, 200, easeOut_AnimFlag);
-                        touch->edgeDragging = NULL;
-                    }
-                }
-                else if (touch->edgeDragging) {
-                    setVisualOffset_Widget(touch->edgeDragging, x_F3(pos) - x_F3(touch->startPos), 10, 0);
-                }
-            }
-            if (touch->edge == right_TouchEdge && fing->dx > 0) {
-                touch->edge = none_TouchEdge;
-            }
-            if (touch->edge) {
-                pixels.y = 0;
-            }
+            iAssert(touch->edge == none_TouchEdge);
             if (touch->axis == x_TouchAxis) {
                 pixels.y = 0;
             }
             if (touch->axis == y_TouchAxis) {
                 pixels.x = 0;
             }
-//            printf("%p (%s) py: %i wy: %f acc: %f\n",
+//            printf("%p (%s) py: %i wy: %f acc: %f edge: %d\n",
 //                   touch->affinity,
 //                   class_Widget(touch->affinity)->name,
-//                   pixels.y, y_F3(amount), y_F3(touch->accum));
+//                   pixels.y, y_F3(amount), y_F3(touch->accum),
+//                   touch->edge);
             if (pixels.x || pixels.y) {
-                setFocus_Widget(NULL);
-                dispatchMotion_Touch_(touch->pos[0], 0);
+                //setFocus_Widget(NULL);
+                dispatchMotion_Touch_(touch->startPos /*pos[0]*/, 0);
                 setCurrent_Root(touch->affinity->root);
                 dispatchEvent_Widget(touch->affinity, (SDL_Event *) &(SDL_MouseWheelEvent){
                     .type = SDL_MOUSEWHEEL,
+                    .which = SDL_TOUCH_MOUSEID,
                     .timestamp = SDL_GetTicks(),
                     .x = pixels.x,
                     .y = pixels.y,
-                    .direction = perPixel_MouseWheelFlag
+                    .direction = perPixel_MouseWheelFlag,
                 });
                 /* TODO: Keep increasing movement if the direction is the same. */
                 clearWidgetMomentum_TouchState_(d, touch->affinity);
@@ -612,9 +644,21 @@ iBool processEvent_Touch(const SDL_Event *ev) {
                 endPinch_TouchState_(d, touch->pinchId);
                 break;
             }
+#if 0
             if (touch->edgeDragging) {
                 setFlags_Widget(touch->edgeDragging, dragged_WidgetFlag, iFalse);
             }
+#endif
+            if (touch->edge && !isStationary_Touch_(touch)) {
+                const iFloat3 gesture = gestureVector_Touch_(touch);
+                const float pixel = window->pixelRatio;
+                const int moveDir = x_F3(gesture) < -pixel ? -1 : x_F3(gesture) > pixel ? +1 : 0;
+                const int didAbort = (touch->edge == left_TouchEdge  && moveDir < 0) ||
+                                     (touch->edge == right_TouchEdge && moveDir > 0);
+                postCommandf_App("edgeswipe.ended abort:%d side:%d id:%llu", didAbort, touch->edge, touch->id);
+                remove_ArrayIterator(&i);
+                continue;
+            }
             if (flags_Widget(touch->affinity) & touchDrag_WidgetFlag) {
                 if (!touch->isLeftDown && !touch->isTapAndHold) {
                     /* This will be a click on a touchDrag widget. */
@@ -638,6 +682,7 @@ iBool processEvent_Touch(const SDL_Event *ev) {
             const uint32_t duration = nowTime - touch->startTime;
             const iFloat3 gestureVector = sub_F3(pos, touch->pos[lastIndex]);
             iFloat3 velocity = zero_F3();
+#if 0
             if (touch->edge && fabsf(2 * x_F3(gestureVector)) > fabsf(y_F3(gestureVector)) &&
                 !isStationary_Touch_(touch)) {
                 const int swipeDir = x_F3(gestureVector) > 0 ? +1 : -1;
@@ -646,7 +691,9 @@ iBool processEvent_Touch(const SDL_Event *ev) {
                                      touch->edge == right_TouchEdge && swipeDir < 0 ? SDL_BUTTON_X2 : 0);
                 setHover_Widget(NULL);
             }
-            else {
+            else
+#endif
+            {
                 const uint32_t elapsed = fing->timestamp - touch->posTime[lastIndex];
                 const float minVelocity = 400.0f;
                 if (elapsed < 150) {
@@ -673,7 +720,7 @@ iBool processEvent_Touch(const SDL_Event *ev) {
                     iMomentum mom = {
                         .affinity = touch->affinity,
                         .releaseTime = nowTime,
-                        .pos = touch->pos[0],
+                        .pos = touch->startPos, // pos[0],
                         .velocity = velocity
                     };
                     if (isEmpty_Array(d->moms)) {
